@@ -14,7 +14,7 @@ Miscellaneous UTF-8 and UTF-16 types and methods.
 * `#![no_std]`-mode: There are a few differences:
   * `AsciiExt` doesn't exist, but `is_ascii()` is made available as an inherent impl.
   * `Error` doesn't exist, but `description()` is made available as an inherent impl.
-  * `Extend`/`FromIterator`-implementations for `String`/`Vec<u8>`/`Vec<u16>`.
+  * `Extend`/`FromIterator`-implementations for `String`/`Vec<u8>`/`Vec<u16>` are missing.
   * There is no `io`, so `Utf8Iterator` doesn't implement `Read`.
 
   This feature is enabled by setting `default-features=false` in `Cargo.toml`:
@@ -24,15 +24,14 @@ Miscellaneous UTF-8 and UTF-16 types and methods.
   [ascii::`AsciiChar`](https://tomprogrammer.github.io/rust-ascii/ascii/enum.AsciiChar.html).
 
 The minimum supported version of Rust is 1.15,
-older versions might work, but also break in a minor update.
+older versions might work now but can break with a minor update.
 
 [crates.io page](https://crates.io/crates/encode_unicode)  
-[githab repository](https://github.com/tormol/encode_unicode)
+[github repository](https://github.com/tormol/encode_unicode)
 
 */
 
-
-#![cfg_attr(feature="std", warn(missing_docs))] // Don't bother documenting std standins.
+#![warn(missing_docs)]
 
 #![cfg_attr(not(feature="std"), no_std)]
 // either `cargo clippy` doesn't see theese, or I get a warning when I build.
@@ -43,6 +42,8 @@ older versions might work, but also break in a minor update.
 #![cfg_attr(feature="clippy", allow(match_same_arms))]// looks better IMO
 #![cfg_attr(feature="clippy", allow(needless_return))]// `foo.bar(); foo` looks unfinished
 #![cfg_attr(feature="clippy", allow(redundant_closure))]// keep it explicit
+#![cfg_attr(feature="clippy", allow(redundant_closure_call))]// not redundant in macros
+#![cfg_attr(feature="clippy", allow(cast_lossless))]// too much noise (and too verbose)
 // precedence: I prefer spaces to parentheses, but it's nice to recheck.
 
 mod errors;
