@@ -18,6 +18,7 @@ use self::core::ops::Deref;
 #[cfg(feature="std")]
 use self::core::iter::FromIterator;
 #[cfg(feature="std")]
+#[allow(deprecated)]
 use std::ascii::AsciiExt;
 #[cfg(feature="ascii")]
 use self::core::char;
@@ -140,6 +141,7 @@ impl Deref for Utf16Char {
  //ascii traits//
 ////////////////
 #[cfg(feature="std")]
+#[allow(deprecated)]
 impl AsciiExt for Utf16Char {
     type Owned = Self;
     fn is_ascii(&self) -> bool {
@@ -268,11 +270,54 @@ impl Utf16Char {
     pub fn len(self) -> usize {
         1 + (self.units[1] as usize >> 15)
     }
-    /// Is this codepoint an ASCII character?
-    #[cfg(not(feature="std"))]
+
+    /// Checks that the codepoint is an ASCII character.
     pub fn is_ascii(&self) -> bool {
         self.units[0] <= 127
     }
+    /// Checks that two characters are an ASCII case-insensitive match.
+    ///
+    /// Is equivalent to `a.to_ascii_lowercase() == b.to_ascii_lowercase()`.
+    #[cfg(feature="std")]
+    pub fn eq_ignore_ascii_case(&self,  other: &Self) -> bool {
+        self.to_ascii_lowercase() == other.to_ascii_lowercase()
+    }
+    /// Converts the character to its ASCII upper case equivalent.
+    ///
+    /// ASCII letters 'a' to 'z' are mapped to 'A' to 'Z',
+    /// but non-ASCII letters are unchanged.
+    #[cfg(feature="std")]
+    pub fn to_ascii_uppercase(&self) -> Self {
+        let n = self.units[0].wrapping_sub(b'a' as u16);
+        if n < 26 {Utf16Char{ units: [n+b'A' as u16, 0] }}
+        else      {*self}
+    }
+    /// Converts the character to its ASCII lower case equivalent.
+    ///
+    /// ASCII letters 'A' to 'Z' are mapped to 'a' to 'z',
+    /// but non-ASCII letters are unchanged.
+    #[cfg(feature="std")]
+    pub fn to_ascii_lowercase(&self) -> Self {
+        let n = self.units[0].wrapping_sub(b'A' as u16);
+        if n < 26 {Utf16Char{ units: [n+b'a' as u16, 0] }}
+        else      {*self}
+    }
+    /// Converts the character to its ASCII upper case equivalent in-place.
+    ///
+    /// ASCII letters 'a' to 'z' are mapped to 'A' to 'Z',
+    /// but non-ASCII letters are unchanged.
+    #[cfg(feature="std")]
+    pub fn make_ascii_uppercase(&mut self) {
+        *self = self.to_ascii_uppercase()
+    }
+    /// Converts the character to its ASCII lower case equivalent in-place.
+    ///
+    /// ASCII letters 'A' to 'Z' are mapped to 'a' to 'z',
+    /// but non-ASCII letters are unchanged.
+    #[cfg(feature="std")]
+    pub fn make_ascii_lowercase(&mut self) {
+        *self = self.to_ascii_lowercase();
+    }
 
     /// Convert from UTF-16 to UTF-32
     pub fn to_char(self) -> char {
