@@ -11,8 +11,11 @@
 
 use std::char;
 use std::str::{self,FromStr};
-use std::hash::Hash;
+use std::cmp::Ordering;
+use std::hash::{Hash,Hasher};
 use std::collections::hash_map::DefaultHasher;
+#[allow(deprecated,unused)]
+use std::ascii::AsciiExt;
 use std::iter::FromIterator;
 extern crate encode_unicode;
 use encode_unicode::*;
@@ -56,44 +59,60 @@ fn read_iterator() {
 }
 
 
-
-const EDGES_AND_BETWEEN: [u32;13] = [
-    0x0,// min
-    0x3b,// between
-    0x7f,// max 1-byte UTF-8
-    0x80,// min 2-byte UTF-8
-    0x111,// between
-    0x7ff,// max 2-byte UTF-8
-    0x800,// min 3-byte UTF-8
-    0xd7ff,// before reserved
-    0xe000,// after reserved
-    0xffff,// max UTF-16 single and 3-byte UTF-8
-    0x10000,// min UTF-16 surrogate and 4-byte UTF-8
-    0xabcde,// between
-    0x10ffff,// max
+const EDGES_AND_BETWEEN: [char;19] = [
+    '\u{0}',// min
+    '\u{3b}',// middle ASCII
+    'A',// min ASCII uppercase
+    'N',// middle ASCII uppercase
+    'Z',// max ASCII uppercase
+    'a',// min ASCII lowercase
+    'm',// middle ASCII lowercase
+    'z',// max ASCII lowercase
+    '\u{7f}',// max ASCII and 1-byte UTF-8
+    '\u{80}',// min 2-byte UTF-8
+    '\u{111}',// middle
+    '\u{7ff}',// max 2-byte UTF-8
+    '\u{800}',// min 3-byte UTF-8
+    '\u{d7ff}',// before reserved
+    '\u{e000}',// after reserved
+    '\u{ffff}',// max UTF-16 single and 3-byte UTF-8
+    '\u{10000}',// min UTF-16 surrogate and 4-byte UTF-8
+    '\u{abcde}',// middle
+    '\u{10ffff}',// max
 ];
 
 fn eq_cmp_hash(c: char) -> (Utf8Char, Utf16Char) {
-    let sh = &mut DefaultHasher::new();
+    fn hash<T:Hash>(v: T) -> u64 {
+        #[allow(deprecated)]
+        let mut hasher = DefaultHasher::new();
+        v.hash(&mut hasher);
+        hasher.finish()
+    }
     let u8c = c.to_utf8();
     assert_eq!(u8c.to_char(), c);
-    assert_eq!(u8c.hash(sh), c.hash(sh));
+    assert_eq!(u8c, u8c);
+    assert_eq!(hash(u8c), hash(u8c));
+    assert_eq!(u8c.cmp(&u8c), Ordering::Equal);
+    assert!(u8c.eq_ignore_ascii_case(&u8c));
     let u16c = c.to_utf16();
     assert_eq!(u16c.to_char(), c);
-    assert_eq!(u16c.hash(sh), c.hash(sh));
-
-    for other in &EDGES_AND_BETWEEN {
-        let other = unsafe{ char::from_u32_unchecked(*other) };
+    assert_eq!(u16c, u16c);
+    assert_eq!(hash(u16c), hash(c));
+    assert_eq!(u16c.cmp(&u16c), Ordering::Equal);
+    assert!(u16c.eq_ignore_ascii_case(&u16c));
 
+    for &other in &EDGES_AND_BETWEEN {
         let u8other = other.to_utf8();
         assert_eq!(u8c == u8other,  c == other);
-        assert_eq!(u8c.hash(sh)==other.hash(sh),  c.hash(sh)==u8other.hash(sh));
+        assert_eq!(hash(u8c)==hash(u8other),  hash(c)==hash(other));
         assert_eq!(u8c.cmp(&u8other), c.cmp(&other));
+        assert_eq!(u8c.eq_ignore_ascii_case(&u8other), c.eq_ignore_ascii_case(&other));
 
         let u16other = other.to_utf16();
         assert_eq!(u16c == u16other,  c == other);
-        assert_eq!(u16c.hash(sh)==other.hash(sh),  c.hash(sh)==u16other.hash(sh));
+        assert_eq!(hash(u16c)==hash(u16other),  hash(c)==hash(other));
         assert_eq!(u16c.cmp(&u16other), c.cmp(&other));
+        assert_eq!(u16c.eq_ignore_ascii_case(&u16other), c.eq_ignore_ascii_case(&other));
     }
     (u8c, u16c)
 }
@@ -118,9 +137,10 @@ fn iterators(c: char) {
     }
 }
 
-fn test(c: u32) {
-    let c = char::from_u32(c).expect(&format!("{:x} is not a valid char", c));
+fn test(c: char) {
+    assert_eq!(char::from_u32(c as u32), Some(c));
     assert_eq!(char::from_u32_detailed(c as u32), Ok(c));
+    assert_eq!(unsafe{ char::from_u32_unchecked(c as u32) }, c);
     let (u8c, u16c) = eq_cmp_hash(c);
     iterators(c);
     assert_eq!(Utf16Char::from(u8c), u16c);
@@ -163,6 +183,9 @@ fn test(c: u32) {
     assert_eq!(&String::from_iter(Some(u8c))[..], str_);
     assert_eq!(format!("{:?}", u8c), format!("{:?}", c));
     assert_eq!(format!("{}", u8c), format!("{}", c));
+    assert_eq!(u8c.is_ascii(), c.is_ascii());
+    assert_eq!(u8c.to_ascii_lowercase().to_char(), c.to_ascii_lowercase());
+    assert_eq!(u8c.to_ascii_uppercase().to_char(), c.to_ascii_uppercase());
 
     // UTF-16
     let mut buf = [0; 2];
@@ -194,13 +217,16 @@ fn test(c: u32) {
     assert_eq!(&Vec::<u16>::from_iter(Some(u16c))[..], reference);
     assert_eq!(format!("{:?}", u16c), format!("{:?}", c));
     assert_eq!(format!("{}", u16c), format!("{}", c));
+    assert_eq!(u16c.is_ascii(), c.is_ascii());
+    assert_eq!(u16c.to_ascii_lowercase().to_char(), c.to_ascii_lowercase());
+    assert_eq!(u16c.to_ascii_uppercase().to_char(), c.to_ascii_uppercase());
 }
 
 
 #[test]
 fn edges_middle() {
-    for c in &EDGES_AND_BETWEEN {
-        test(*c);
+    for &c in &EDGES_AND_BETWEEN {
+        test(c);
     }
 }
 
@@ -208,7 +234,8 @@ fn edges_middle() {
 #[test]
 #[ignore]
 fn all() {
-    for c in std::iter::Iterator::chain(0..0xd800, 0xe000..0x110000) {
+    for cp in std::iter::Iterator::chain(0..0xd800, 0xe000..0x110000) {
+        let c = char::from_u32(cp).expect("not a valid char");
         test(c);
     }
 }
